//------------------------------------------------------------
/// \file	Main.cpp
/// \author	Rob Bateman
/// \date	9-feb-2005
/// \brief	This example uses glBlendFunc to use an alpha map
///			based transparency for textures. 
//------------------------------------------------------------

#include <stdlib.h>
#include <iostream>
#include <GL/glut.h>
#include "texture.h"

/// this will hold the openGL texture object for the loaded texture
unsigned int g_Texture=0;

//------------------------------------------------------------	Reshape()
void OnReshape(int w, int h)
{
	if (h==0) {
		h=1;
	}
	glViewport(0,0,w,h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-20,20,20,-20,-20,20);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

//------------------------------------------------------------	Draw()
//
void OnDraw() {

	glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT);

	glLoadIdentity();

	// apply texture
	glBindTexture(GL_TEXTURE_2D,g_Texture);

	// draw 2D quad
	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex2f(-15,15);

		glTexCoord2f(0,1);
		glVertex2f(-15,-15);

		glTexCoord2f(1,1);
		glVertex2f(15,-15);

		glTexCoord2f(1,0);
		glVertex2f(15,15);
	glEnd();

	glutSwapBuffers();
}

//------------------------------------------------------------	OnInit()
//
void OnInit() {

	// set the background oolour
	glClearColor(0.5,0.2,0.4,0.0);

	// 
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);

	// set the blending equation to use the alpha channel to blend
	// the texture and whats already in the framebuffer. In this
	// case we use the following source & destination parameters :
	// 
	// SRC	=	GL_SRC_ALPHA
	// DST	=	GL_ONE_MINUS_SRC_ALPHA
	// 
	// To calculate the final pixel color it then uses :
	//
	// OUTPUT = SRC * IN_COLOR  +  DST * FRAME_BUFFER_COLOR
	// 
	// In effect, where the alpha value is 0, the frame buffer colour
	// will be shown - ie, 0=transparent. When we have an alpha value
	// of 1, then the input colour will be used. 
	// 
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);

	// load texture as compressed RGBA
	g_Texture = LoadTexture("blend.tga");

	std::cout << "Texture Size=" << (GetTextureSize(g_Texture)/1024.0f) << "Kb"<< std::endl;
}

//------------------------------------------------------------	OnExit()
//
void OnExit() {
	DeleteAllTextures();
}

//------------------------------------------------------------	main()
//
int main(int argc,char** argv) {

	glutInit(&argc,argv);
	glutInitDisplayMode(GLUT_DEPTH|GLUT_RGBA|GLUT_DOUBLE);
	glutInitWindowSize(640,480);
	glutCreateWindow("Texture Transparency Example");

	glutDisplayFunc(OnDraw);
	glutReshapeFunc(OnReshape);
	
	OnInit();

	atexit(OnExit);

	glutMainLoop();
	return 0;
}
